#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Server_1 = require("./Server");
const services = require("./services");
const logger = require("./shared/Logger");
const program = require("commander");
const path = require("path");
class Cli {
    constructor() {
        this.program = new program.Command();
        this.serverConfig = {
            dataPath: path.resolve("data")
        };
    }
    start(argv) {
        this.program.version(require('../package.json').version);
        this.program
            .command('start')
            .description('start Realm Object Server')
            .option("-d, --data <path>", "Specify the data path to use", "./data")
            .option("-a, --address <address>", "Specify the address to listen on", "0.0.0.0")
            .option("-p, --port <port>", "Specify the port to listen on", 9080)
            .option("-l, --loglevel <level>", "The log level (all, trace, debug, detail, info, warm, error, fata, off)", "info")
            .option("-f, --logfile <filename>", "The file to log to, or - for stdout", "-")
            .option("--no-auto-keygen", "Do not autogenerate keys on startup")
            .option("--private-key <privateKeyPath>", "Alternative path to the private key")
            .option("--public-key <publicKeyPath>", "Alternative path to the public key")
            .option("--refresh-token-ttl <refreshTokenTtl>", "TTL of issued refresh tokens in seconds", 10 * 60 * 60 * 24 * 365 * 10)
            .option("--access-token-ttl <accessTokenTtl>", "TTL of issued access tokens in seconds", 10 * 60)
            .option('--auth <provider1[,provider2 ...]>', "A list of auth providers to load", "password")
            .action((command) => {
            const config = {
                dataPath: path.resolve(command.data),
                privateKeyPath: command.privateKey ? path.resolve(command.privateKey) : null,
                publicKeyPath: command.publicKey ? path.resolve(command.publicKey) : null,
                autoKeyGen: command.autoKeygen,
                hostname: command.address,
                port: command.port,
            };
            if (command.logfile !== "-") {
                config.logger = new logger.File(command.logfile, command.loglevel);
            }
            else {
                config.logger = new logger.Console(command.loglevel);
            }
            const server = new Server_1.Server(config);
            const authService = new services.AuthService({ refreshTokenTtl: command.refreshTokenTtl });
            const providerNames = command.auth.split(',');
            for (const providerName of providerNames) {
                const providerClass = require(`./auth/${providerName}`).default;
                if (providerClass) {
                    const options = {};
                    if (providerName === "password") {
                        options.autoCreateAdminUser = true;
                    }
                    const provider = new providerClass(options);
                    authService.addProvider(provider);
                }
                else {
                    throw new Error(`default export not defined on auth provider '${providerName}'`);
                }
            }
            server.addService(authService);
            server.addService(new services.RealmDirectoryService({ accessTokenTtl: command.accessTokenTtl }));
            server.addService(new services.LogService());
            server.addService(new services.SyncProxyService());
            server.addService(new services.SyncService());
            server.addService(new services.HealthService());
            process.removeAllListeners("SIGINT");
            process.on('SIGINT', () => __awaiter(this, void 0, void 0, function* () {
                console.log("Got signal SIGINT.  Shutting down.");
                server.shutdown()
                    .then(() => process.exit(0))
                    .catch((err) => {
                    console.error(err);
                    process.exit(1);
                });
            }));
            process.removeAllListeners("SIGTERM");
            process.on('SIGTERM', () => __awaiter(this, void 0, void 0, function* () {
                console.log("Got signal SIGTERM.  Shutting down.");
                server.shutdown()
                    .then(() => process.exit(0))
                    .catch((err) => {
                    console.error(err);
                    process.exit(1);
                });
            }));
            server.start().catch(err => console.error(err));
        });
        this.program
            .command('init')
            .description('Initialize a new ROS project')
            .action(() => {
            console.error("init: not yet implemented");
            process.exit(1);
        });
        if (!argv.slice(2).length) {
            this.program.outputHelp();
            process.exit(1);
        }
        return this.program.parse(argv);
    }
}
exports.Cli = Cli;
if (!module.parent) {
    new Cli().start(process.argv);
}
//# sourceMappingURL=cli.js.map