"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const TestServer_1 = require("../TestServer");
const services = require("../services");
const auth = require("../auth");
const chai_1 = require("chai");
const chai = require("chai");
chai.use(require("chai-as-promised"));
const superagent = require("superagent");
const faker = require("faker");
describe('Password Auth Provider Integration Tests', function () {
    let server;
    let authService;
    let provider;
    before(() => __awaiter(this, void 0, void 0, function* () {
        server = new TestServer_1.TestServer();
        server.addService(new services.SyncService());
        authService = new services.AuthService();
        provider = new auth.PasswordAuthProvider({ saltWorkIterations: 1 });
        authService.addProvider(provider);
        authService.addProvider(new auth.DebugAuthProvider());
        server.addService(authService);
        yield server.start();
    }));
    after(() => __awaiter(this, void 0, void 0, function* () {
        yield server.shutdown();
    }));
    describe("POST /auth (authenticateOrCreateUser)", () => {
        describe("when username is undefined", () => {
            it("should reject with an error", () => __awaiter(this, void 0, void 0, function* () {
                yield chai_1.assert.isRejected(superagent.post(`${server.url}/auth`).send({
                    provider: "password",
                    password: "some password",
                }), "Bad Request");
            }));
        });
        describe("when password is undefined", () => {
            it("should reject with an error", () => __awaiter(this, void 0, void 0, function* () {
                yield chai_1.assert.isRejected(superagent.post(`${server.url}/auth`).send({
                    provider: "password",
                    username: "Emmanuel.Sanders",
                }), "Bad Request");
            }));
        });
        describe("when register is undefined", () => {
            describe("without an existing user", () => {
                it("should reject with an error", () => __awaiter(this, void 0, void 0, function* () {
                    const username = faker.internet.userName();
                    const password = faker.internet.password();
                    const response = yield chai_1.assert.isRejected(superagent.post(`${server.url}/auth`).send({
                        provider: "password",
                        username, password,
                    }), "Unauthorized");
                }));
            });
            describe("with an existing user", () => {
                let username;
                let password;
                before(() => __awaiter(this, void 0, void 0, function* () {
                    username = faker.internet.userName();
                    password = faker.internet.password();
                    yield superagent.post(`${server.url}/auth`).send({
                        provider: "password",
                        username, password,
                        register: true,
                    });
                }));
                describe("with bad password", () => {
                    it("should reject with an error", () => __awaiter(this, void 0, void 0, function* () {
                        const error = yield chai_1.assert.isRejected(superagent.post(`${server.url}/auth`).send({
                            provider: "password",
                            username,
                            password: "tombrady",
                        }), "Unauthorized");
                        const response = error.response;
                        chai_1.assert.equal(response.status, 401);
                        chai_1.assert.equal(response.body.code, 611);
                    }));
                });
                describe("with correct password", () => {
                    it("should reject with an error", () => __awaiter(this, void 0, void 0, function* () {
                        const response = yield chai_1.assert.isFulfilled(superagent.post(`${server.url}/auth`).send({
                            provider: "password",
                            username, password,
                        }));
                        chai_1.assert.equal(response.status, 200);
                        const refreshToken = response.body.refresh_token;
                        chai_1.assert.isDefined(refreshToken);
                        chai_1.assert.isString(refreshToken.token);
                        const data = refreshToken.token_data;
                        chai_1.assert.isFalse(data.is_admin);
                        chai_1.assert.isDefined(data.identity);
                        chai_1.assert.deepEqual(data.access, ["refresh"]);
                        chai_1.assert.isNumber(data.expires);
                        chai_1.assert.isDefined(data.app_id);
                    }));
                });
            });
        });
        describe("when register = true", () => {
            describe("with an existing user", () => {
                let username;
                let password;
                before(() => __awaiter(this, void 0, void 0, function* () {
                    username = faker.internet.userName();
                    password = faker.internet.password();
                    yield superagent.post(`${server.url}/auth`).send({
                        provider: "password",
                        username, password,
                        register: true,
                    });
                }));
                it("should reject with an error", () => __awaiter(this, void 0, void 0, function* () {
                    const error = yield chai_1.assert.isRejected(superagent.post(`${server.url}/auth`).send({
                        provider: "password",
                        username, password,
                        register: true,
                    }), "Unauthorized");
                    const response = error.response;
                    chai_1.assert.equal(response.status, 401);
                    chai_1.assert.equal(response.body.code, 611);
                }));
            });
            describe("without an existing user", () => {
                it("should create a user", () => __awaiter(this, void 0, void 0, function* () {
                    const username = faker.internet.userName();
                    const password = faker.internet.password();
                    const response = yield chai_1.assert.isFulfilled(superagent.post(`${server.url}/auth`).send({
                        provider: "password",
                        username, password,
                        register: true,
                    }));
                    const refreshToken = response.body.refresh_token;
                    chai_1.assert.isDefined(refreshToken);
                    chai_1.assert.isString(refreshToken.token);
                    yield chai_1.assert.isFulfilled(superagent.post(`${server.url}/auth`).send({
                        provider: "password",
                        username, password
                    }));
                }));
            });
        });
    });
    describe("PUT /auth/password (update)", () => {
        let existingUsername;
        let existingPassword;
        let existingUserToken;
        let existingUserId;
        beforeEach(() => __awaiter(this, void 0, void 0, function* () {
            existingUsername = faker.internet.userName();
            existingPassword = faker.internet.password();
            const response = yield superagent.post(`${server.url}/auth`).send({
                provider: "password",
                username: existingPassword,
                password: existingPassword,
                register: true,
            });
            existingUserToken = response.body.refresh_token.token;
            existingUserId = response.body.refresh_token.token_data.identity;
        }));
        describe("as an existing user", () => {
            describe("when newPassword is not defined", () => {
                it("should reject with an error", () => __awaiter(this, void 0, void 0, function* () {
                    yield chai_1.assert.isRejected(superagent.put(`${server.url}/auth/password`).set({
                        Authorization: existingUserToken,
                    }).send({
                        data: {}
                    }), "Bad Request");
                }));
            });
            describe("when using newPassword", () => {
                it("should set the password", () => __awaiter(this, void 0, void 0, function* () {
                    yield chai_1.assert.isFulfilled(superagent.put(`${server.url}/auth/password`).set({
                        Authorization: existingUserToken,
                    }).send({
                        data: {
                            newPassword: "new-password",
                        }
                    }));
                }));
            });
        });
        describe("as an admin", () => {
            let adminUserToken;
            before(() => __awaiter(this, void 0, void 0, function* () {
                const response = yield superagent.post(`${server.url}/auth`).send({
                    provider: "debug",
                    data: "admin"
                });
                adminUserToken = response.body.refresh_token.token;
            }));
            it("should set the password", () => __awaiter(this, void 0, void 0, function* () {
                yield chai_1.assert.isFulfilled(superagent.put(`${server.url}/auth/password`).set({
                    Authorization: adminUserToken,
                }).send({
                    userId: existingUserId,
                    data: {
                        newPassword: "new-password",
                    }
                }));
            }));
        });
        describe("without a token", () => {
            it("should reject with an error", () => __awaiter(this, void 0, void 0, function* () {
                yield chai_1.assert.isRejected(superagent.put(`${server.url}/auth/password`).send({
                    userId: existingUserId,
                    data: {
                        newPassword: "new-password",
                    }
                }), "Forbidden");
            }));
        });
    });
});
//# sourceMappingURL=password-auth-integration-tests.spec.js.map