"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Realm = require("realm");
const path = require("path");
const superagent = require("superagent");
const URI = require("urijs");
const crypto = require("crypto");
const decorators_1 = require("../decorators");
const Server_1 = require("../Server");
const SyncedRealm_1 = require("../shared/SyncedRealm");
const Token_1 = require("../shared/Token");
const errors = require("../errors");
const util_1 = require("../shared/util");
class User {
    constructor() {
        this.created = false;
    }
    toJSON() {
        return {
            user_id: this.userId,
            is_admin: this.isAdmin,
            accounts: this.accounts.slice().map((a) => a.toJSON()),
            metadata: this.metadata.slice(),
        };
    }
}
User.schema = {
    name: 'User',
    primaryKey: 'userId',
    properties: {
        userId: { type: 'string', optional: false },
        isAdmin: { type: 'bool', optional: false, default: false },
        accounts: { type: "list", objectType: "Account" },
        metadata: { type: 'list', objectType: 'UserMetadataRow' }
    }
};
exports.User = User;
class Account {
    toJSON() {
        return {
            provider: this.provider,
            provider_id: this.providerId,
        };
    }
}
Account.schema = {
    name: 'Account',
    properties: {
        provider: { type: 'string', optional: false, indexed: true },
        providerId: { type: "string", optional: false, indexed: true },
        user: { type: 'linkingObjects', objectType: 'User', property: 'accounts' }
    }
};
exports.Account = Account;
class Metadata {
}
Metadata.schema = {
    name: 'UserMetadataRow',
    properties: {
        user: { type: 'linkingObjects', objectType: 'User', property: 'metadata' },
        key: { type: 'string', optional: false },
        value: { type: 'string', optional: false }
    }
};
exports.Metadata = Metadata;
class TokenRevocation {
}
TokenRevocation.schema = {
    name: 'TokenRevocation',
    primaryKey: 'token',
    properties: {
        expires: { type: "date" },
        revoked: { type: "date" },
        token: "string",
    }
};
exports.TokenRevocation = TokenRevocation;
let AuthService = class AuthService {
    constructor(config = {}) {
        this.providers = [];
        this.refreshTokenTtl = config.refreshTokenTtl || 315360000;
    }
    get adminRealm() {
        return this.syncedAdminRealm.realm;
    }
    get revocationRealm() {
        return this.syncedRevocationRealm.realm;
    }
    addProvider(provider) {
        this.providers.push(provider);
    }
    start(server) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger = server.logger.withContext({ service: "auth" });
            this.discovery = server.discovery;
            this.privateKey = server.privateKey;
            this.publicKey = server.publicKey;
            this.dataPath = path.resolve(server.config.dataPath, "auth");
            this.syncedAdminRealm = new SyncedRealm_1.SyncedRealm(this.discovery, {
                path: path.resolve(this.dataPath, "admin.realm"),
                schema: [util_1.createRealmSchema(User), util_1.createRealmSchema(Account), util_1.createRealmSchema(Metadata)],
                sync: {
                    url: "realm://default/__admin",
                    user: Realm.Sync.User.adminUser(Token_1.generateAdminToken(this.privateKey), "realm://default/__admin"),
                }
            });
            this.syncedRevocationRealm = new SyncedRealm_1.SyncedRealm(this.discovery, {
                path: path.resolve(server.config.dataPath, "auth", "__revocation.realm"),
                schema: [util_1.createRealmSchema(TokenRevocation)],
                sync: {
                    url: "realm://default/__revocation",
                    user: Realm.Sync.User.adminUser(Token_1.generateAdminToken(this.privateKey), "realm://default/__revocation"),
                }
            });
            this.pruneRevocationTokens();
            this.cleanupInterval = setInterval(this.pruneRevocationTokens.bind(this), 1000 * 60 * 60 * 24);
            for (const provider of this.providers) {
                this.logger.info(`Starting auth provider '${provider.name}'`);
                Reflect.set(provider, "service", this);
                if (provider.start) {
                    yield provider.start();
                }
            }
        });
    }
    stop() {
        return __awaiter(this, void 0, void 0, function* () {
            for (const provider of this.providers) {
                if (provider.stop) {
                    yield provider.stop();
                }
                Reflect.set(provider, "service", undefined);
            }
            if (this.syncedAdminRealm) {
                this.syncedAdminRealm.close();
            }
            if (this.cleanupInterval) {
                clearInterval(this.cleanupInterval);
            }
            if (this.syncedRevocationRealm) {
                this.syncedRevocationRealm.close();
            }
        });
    }
    authenticate(request) {
        return __awaiter(this, void 0, void 0, function* () {
            const providerName = request.body.provider;
            if (!providerName) {
                throw new errors.realm.MissingParameters('provider');
            }
            if (providerName === 'realm') {
                const data = request.body.data;
                if (!data) {
                    throw new errors.realm.MissingParameters('data');
                }
                let refreshToken;
                try {
                    refreshToken = Token_1.Token.parse(data, this.publicKey);
                }
                catch (err) {
                    throw new errors.realm.AccessDenied();
                }
                if (refreshToken.access.indexOf("refresh") === -1) {
                    throw new errors.realm.AccessDenied();
                }
                const foundRevocation = this.revocationRealm.objectForPrimaryKey('TokenRevocation', refreshToken.encodedData());
                if (foundRevocation) {
                    throw new errors.realm.AccessDenied();
                }
                let handle = yield this.discovery.find('realms');
                if (!handle) {
                    throw new Error('Unable to contact the RealmDirectoryService with the name: realms');
                }
                const accessTokenUrl = new URI(`http://${handle.address}:${handle.port}/`)
                    .segment('/realms')
                    .segment('/access_token').toString();
                const response = yield superagent.post(accessTokenUrl)
                    .send(request.body);
                return response.body;
            }
            const provider = this.providers.find((p) => { return p.name === providerName; });
            if (!provider) {
                throw new errors.realm.InvalidParameters('provider');
            }
            const user = yield provider.authenticateOrCreateUser(request.body);
            const appId = request.body["app_id"] || 'io.realm.Auth';
            const refreshToken = new Token_1.RefreshToken({
                appId,
                identity: user.userId,
                isAdmin: user.isAdmin,
                expires: Math.round(Date.now() / 1000) + this.refreshTokenTtl,
            });
            return {
                refresh_token: {
                    token: refreshToken.sign(this.privateKey),
                    token_data: refreshToken.toJSON(),
                }
            };
        });
    }
    revokeToken(tokenData, req) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!tokenData || tokenData === "") {
                throw new errors.realm.MissingParameters('token');
            }
            const token = Token_1.Token.parse(tokenData, this.publicKey);
            this.requireAdminOrUserId(req, token.identity);
            const expires = token.expires ? new Date(token.expires * 1000) : false;
            const revoked = new Date();
            if (expires && expires > revoked) {
                this.revocationRealm.write(() => {
                    this.revocationRealm.create('TokenRevocation', {
                        expires,
                        token: token.encodedData(),
                        revoked,
                    }, true);
                });
            }
            return {};
        });
    }
    getUserById(userId, req) {
        this.requireAdminOrUserId(req, userId);
        const user = this.adminRealm.objectForPrimaryKey('User', userId);
        if (!user && req) {
            throw new errors.realm.UnknownAccount();
        }
        return user;
    }
    getUserByProviderId(provider, providerId, req) {
        this.requireAdminOrUserId(req);
        const decodedProviderId = URI.decode(providerId);
        const accounts = this.adminRealm.objects('Account').filtered(`provider = $0 AND providerId = $1`, provider, decodedProviderId);
        if (accounts.length > 0) {
            const user = accounts[0].user[0];
            if (user) {
                return user;
            }
        }
        if (req) {
            throw new errors.realm.UnknownAccount();
        }
    }
    createOrUpdateUser(providerId, provider, isAdmin, metadata = {}, userId, req) {
        return __awaiter(this, void 0, void 0, function* () {
            this.requireAdminOrUserId(req);
            const canSetAdmin = !req || (req.authToken && req.authToken.isAdmin);
            if (!providerId && !provider) {
                throw new errors.realm.MissingParameters('providerId', 'provider');
            }
            if (!provider && providerId) {
                throw new errors.realm.MissingParameters('provider');
            }
            if (provider && !providerId) {
                throw new errors.realm.MissingParameters('providerId');
            }
            if (userId) {
                const encodedUserId = encodeURIComponent(userId);
                if (userId !== encodedUserId) {
                    throw new errors.realm.InvalidParameters({
                        name: 'userId',
                        reason: 'Realm requires the userId that does not require URI encoding.'
                    });
                }
            }
            let user = this.getUserByProviderId(provider, providerId);
            if (user) {
                if (userId && userId !== user.userId) {
                    throw new errors.realm.InvalidParameters({
                        name: 'userId',
                        reason: "userId does not match the user found by provider/providerId"
                    });
                }
                else if (!userId) {
                    userId = user.userId;
                }
            }
            this.adminRealm.write(() => {
                if (!user) {
                    user = this.adminRealm.create('User', {
                        userId: userId || this.generateUniqueId(),
                        isAdmin: canSetAdmin ? (isAdmin || false) : false,
                    });
                    user.accounts.push({
                        provider: provider,
                        providerId: providerId
                    });
                    user.created = true;
                }
                else {
                    user.created = false;
                }
                if (canSetAdmin && isAdmin !== undefined) {
                    user.isAdmin = isAdmin;
                }
                this.adminRealm.delete(user.metadata);
                for (let key in metadata) {
                    user.metadata.push({
                        key: key,
                        value: metadata[key]
                    });
                }
            });
            return user;
        });
    }
    updateProviderData(providerName, userId, data, req) {
        return __awaiter(this, void 0, void 0, function* () {
            let changeUserId;
            if (req) {
                if (req.authToken) {
                    this.requireAdminOrUserId(req, req.authToken.identity);
                    changeUserId = req.authToken.identity;
                    if (req.authToken.isAdmin && userId) {
                        changeUserId = userId;
                    }
                }
                else {
                    throw new errors.realm.AccessDenied();
                }
            }
            else {
                changeUserId = userId;
            }
            const provider = this.providers.find((p) => { return p.name === providerName; });
            if (!provider) {
                throw new errors.realm.InvalidParameters('provider');
            }
            const user = this.getUserById(changeUserId);
            if (!user) {
                throw new errors.realm.UnknownAccount();
            }
            if (provider.update) {
                return provider.update(user, data);
            }
            else {
                return {};
            }
        });
    }
    pruneRevocationTokens() {
        const deadRevocations = this.revocationRealm.objects("TokenRevocation").filtered("expires < $0", new Date());
        if (deadRevocations.length > 0) {
            this.revocationRealm.write(() => {
                this.revocationRealm.delete(deadRevocations);
            });
        }
    }
    generateUniqueId() {
        return crypto.randomBytes(16).toString('hex');
    }
    checkNotRevoked(token) {
        const foundRevocation = this.revocationRealm.objectForPrimaryKey('TokenRevocation', token.encodedData());
        if (foundRevocation) {
            throw new errors.realm.AccessDenied();
        }
    }
    requireAdminOrUserId(req, userId) {
        if (req) {
            if (!req.authToken) {
                throw new errors.realm.AccessDenied();
            }
            this.checkNotRevoked(req.authToken);
            if (!req.authToken.isAdmin) {
                if (userId && userId !== req.authToken.identity) {
                    throw new errors.realm.AccessDenied();
                }
                else if (!userId) {
                    throw new errors.realm.AccessDenied();
                }
            }
        }
    }
};
__decorate([
    decorators_1.Start(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Server_1.Server]),
    __metadata("design:returntype", Promise)
], AuthService.prototype, "start", null);
__decorate([
    decorators_1.Stop(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], AuthService.prototype, "stop", null);
__decorate([
    decorators_1.Post('/'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], AuthService.prototype, "authenticate", null);
__decorate([
    decorators_1.Post("/revoke"),
    __param(0, decorators_1.Body('token')),
    __param(1, decorators_1.Request()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, Object]),
    __metadata("design:returntype", Promise)
], AuthService.prototype, "revokeToken", null);
__decorate([
    decorators_1.Get('/users/:user_id'),
    __param(0, decorators_1.Params('user_id')),
    __param(1, decorators_1.Request()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, Object]),
    __metadata("design:returntype", User)
], AuthService.prototype, "getUserById", null);
__decorate([
    decorators_1.Get('/users/:provider/:provider_id'),
    __param(0, decorators_1.Params('provider')),
    __param(1, decorators_1.Params('provider_id')),
    __param(2, decorators_1.Request()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String, Object]),
    __metadata("design:returntype", User)
], AuthService.prototype, "getUserByProviderId", null);
__decorate([
    decorators_1.Put('/users'),
    __param(0, decorators_1.Body('provider_id')),
    __param(1, decorators_1.Body('provider')),
    __param(2, decorators_1.Body('is_admin')),
    __param(3, decorators_1.Body('metadata')),
    __param(4, decorators_1.Body('user_id')),
    __param(5, decorators_1.Request()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String, Boolean, Object, String, Object]),
    __metadata("design:returntype", Promise)
], AuthService.prototype, "createOrUpdateUser", null);
__decorate([
    decorators_1.Put('/:provider'),
    __param(0, decorators_1.Params('provider')),
    __param(1, decorators_1.Body('user_id')),
    __param(2, decorators_1.Body('data')),
    __param(3, decorators_1.Request()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String, Object, Object]),
    __metadata("design:returntype", Promise)
], AuthService.prototype, "updateProviderData", null);
AuthService = __decorate([
    decorators_1.BaseRoute('/auth'),
    decorators_1.ServiceName('auth'),
    __metadata("design:paramtypes", [Object])
], AuthService);
exports.AuthService = AuthService;
//# sourceMappingURL=AuthService.js.map